package de.kebidge.camunda;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.http.HttpClient.Redirect;
import java.net.http.HttpClient.Version;
import java.net.http.HttpResponse.BodyHandlers;
import java.time.Duration;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletResponse;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;

public class SymbolSearchDelegate implements JavaDelegate {

    private final static Logger LOGGER = Logger.getLogger("de.kebidge.camunda.SymbolDelegate");

    // execute ServiceTask_1 (Get Symbol)
    public void execute(DelegateExecution execution) throws Exception {

        // process variables
        //String stock_process = "stock";
        String stock_value = (String) execution.getVariable("stock");
        String symbol_process = "symbol";
        String symbol_value = "";

        // set alphavantage key here
        // https://www.alphavantage.co/support/#api-key
        String key = "VPNHOQPE0AOJ8BEP";

        LOGGER.info("Processing request by '" + stock_value + "'...");

        // build GET Request
        // GET https://www.alphavantage.co/query?function=SYMBOL_SEARCH&keywords=IBM&apikey=VPNHOQPE0AOJ8BEP
        String apiURL = "https://www.alphavantage.co/";
        String queryEndpoint = "query?function=SYMBOL_SEARCH&keywords=";
        String apiKey = "&apikey=" + key;

        String uri_string = apiURL + queryEndpoint + stock_value + apiKey;

        // set HTTP Client
        HttpClient vClient = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(20))
                .followRedirects(Redirect.NORMAL).version(Version.HTTP_1_1).build();

        // set HTTP Request
        HttpRequest vRequest = HttpRequest.newBuilder().headers("Content-Type", "application/json")
                .timeout(Duration.ofSeconds(20))
                .uri(URI.create(uri_string)).build();

        LOGGER.info("Build GET Request: " + uri_string);

        try {
            HttpResponse<String> vResponse = vClient.send(vRequest, BodyHandlers.ofString());

            // check requested resource is available
            if (vResponse.statusCode() == HttpServletResponse.SC_NOT_FOUND) {
                
                // requested resource is not available (404)
                // set null to process variable
                execution.setVariable(symbol_process, null);                
                LOGGER.info("SC_NOT_FOUND -> set variable symbol to null");
            } else {
                
                // requested resource is available (200)

                // get symbol value from REST API
                symbol_value = vResponse.body();

                // set symbol value to process variable
                execution.setVariable(symbol_process, symbol_value);

                // log success and symbol value
                LOGGER.info("HttpServletResponse succeed...");
                //LOGGER.info("Set variable symbol = " + symbol_value);
            }
        } catch (IOException | InterruptedException e) {
                
            // IOException
            // set null to process variable
            execution.setVariable(symbol_process, null);
            LOGGER.info("CATCH IOException -> set variable symbol to null");
        }
    }
}